#include<stdio.h>
#include<math.h>
#include<fonction.h>
#include<constant.h>
#include<dimension.h>
#include<structure.h>

/****************************************************************/
/*		nom:		unmag				*/
/*		auteur:		Jean-Paul Kneib			*/
/*		date:		10/02/92			*/
/*		place:		Toulouse			
 *****************************************************************
 * Return an ellipse that contains the eigenvalues of the amplification 
 * matrix [ a=(1-k+gamma), b=(1-k-gamma) ] and the orientation of the proper 
 * magnification axis.
 * You have to multiply those eigenvalues by c2/2/DOL to get the true
 * eigenvalues.
 * 
 * Here :
 * k = 0.5 * DLS/DS * ( d2Phixx + d2Phiyy )
 * gamma = DLS/DS * SQRT ( 0.25 * (d2Phiyy - d2Phixx)^2 + (d2Phixy)^2 )
 * 
 * Parameters :
 * - position is the point where the amplification matrix is computed
 * - dlsds is the lens efficacity ratio DLS/DS
 * 
 * Global variables used :
 * - in e_grad2() : G, lens, lens_table
 */
struct	ellipse	e_unmag(struct point *position,double dlsds)
{
	struct	ellipse	ampli;
	double	A,B,C;
	struct	matrix	M;
	
	M=e_grad2(position);
	M.a*=dlsds;
	M.b*=dlsds;
	M.c*=dlsds;
//	M.d*=dlsds;
	A=1.-M.a;   // 1 - DLS/DS * d2phixx
	B= -M.b;    // - DLS/DS * d2phixy
	C=1.-M.c;   // 1 - DLS/DS * d2phiyy
	ampli=formeli(A,B,C);
	
	
	return(ampli);
}

struct	ellipse	e_unmag_gal(struct galaxie *image,double dlsds)
{
	struct	ellipse	ampli;
	double	A,B,C;
	struct	matrix	M;
	
	M=e_grad2_gal(image);
	M.a*=dlsds;
	M.b*=dlsds;
	M.c*=dlsds;
	M.d*=dlsds;
	A=1.-M.a;
	B= -M.b;
	C=1.-M.c;
	ampli=formeli(A,B,C);
	
	
	return(ampli);
}
