#include<stdio.h>
#include<math.h>
#include<fonction.h>
#include<constant.h>
#include<dimension.h>
#include<structure.h>
#include<lt.h>


/****************************************************************/ 
/*		nom:		g_shear				*/ 
/*		auteur:		Jean-Paul Kneib			*/
/*		date:		10/02/92			*/
/*		place:		Toulouse			*/
/****************************************************************/

void	g_shear(int ishear,int np,double z,char *file)
{
	extern  struct  g_mode          M;
	extern	struct	g_frame		F;
	extern	struct	pot	lens[];
	
	register int	i,j,ii,jj;
	double	q,dlsds;
	struct	point	ps,pi;
	struct	ellipse	ampli;
	double	**shear;
	int	**nsh;
	double	dx,dy,sxmin,sxmax,symin,symax;
	
	if (ishear==1)
	{
		NPRINTF(stderr,"COMP:shear_map (gamma) in the Image Plane for z_s=%.3lf =>%s\n",z,file);
	}	
	else if (ishear==2)
	{
		NPRINTF(stderr,"COMP:shear_map (eps) in the Image Plane for z_s=%.3lf=>%s\n",
			z,file);
	}
	else if (ishear==-1)
	{
		NPRINTF(stderr,"COMP:shear_map (gamma) in the Source Plane for z_s=%.3lf =>%s\n",
			z,file);
	}
	else if (ishear==-2)
	{
		NPRINTF(stderr,"COMP:shear_map (eps) in the Source Plane for z_s=%.3lf =>%s\n",
			z,file);
	}
	
	dlsds=dratio(lens[0].z,z);
	
	shear=(double **) alloc_square_double(np,np);
	nsh=(int **) alloc_square_int(np,np);
	
	if (ishear>0)
	{
		for(j=0;j<np;j++)
		{
			pi.y=j*(F.ymax-F.ymin)/(np-1)+F.ymin;
			for(i=0;i<np;i++)
			{
				pi.x=i*(F.xmax-F.xmin)/(np-1)+F.xmin;
				ampli=e_unmag(&pi,dlsds);
				if ( ishear==1)
					shear[j][i]=(ampli.a-ampli.b)/2.;
				else if ( ishear==2)
				{
					q=ampli.a/ampli.b;
					shear[j][i]=fabs((q*q-1.)/(q*q+1.));
				}
			};
		};

		for(ii=0;ii<np;ii++)
			for(jj=0;jj<np;jj++)
				if (nsh[jj][ii]>0)
					shear[jj][ii] /=nsh[jj][ii];

		if (M.iref>0)
		{	
			wrf_fits_abs(file,shear,np,np,F.xmin,F.xmax,F.ymin,F.ymax,M.ref_ra,M.ref_dec);
		}
		else
		{
			wrf_fits(file,shear,np,np,F.xmin,F.xmax,F.ymin,F.ymax);
		}
		
	}
	else
	{
		dx=(F.xmax-F.xmin)/6.;
		dy=(F.ymax-F.ymin)/6.;
		sxmin=F.xmin+dx;	/*define a smaller window in the Source plane*/
		sxmax=F.xmax-dx;
		symin=F.ymin+dy;
		symax=F.ymax-dy;
		dx=(sxmax-sxmin)/(np-1);
		dy=(symax-symin)/(np-1);
	
		for(j=0;j<(np*1.8);j++)
		{
			pi.y=j*(F.ymax-F.ymin)/(np*1.8-1)+F.ymin;
			for(i=0;i<(np*1.8);i++)
			{
				pi.x=i*(F.xmax-F.xmin)/(np*1.8-1)+F.xmin;
				ampli=e_unmag(&pi,dlsds);
				e_dpl(&pi,dlsds,&ps);
				ii=(int) (0.5+(ps.x-sxmin)/dx);
				jj=(int) (0.5+(ps.y-symin)/dy);
			
				if ((ii>=0) && (ii<np) && (jj>=0) && (jj<np))
				{
					if ( ishear==-1)
						shear[jj][ii] +=(ampli.a-ampli.b)/2.;
					else if ( ishear==-2)
					{
						q=ampli.a/ampli.b;
						shear[jj][ii] +=fabs((q*q-1.)/(q*q+1.));
					}
					nsh[jj][ii]++;
				}
			}
		}

		for(ii=0;ii<np;ii++)
			for(jj=0;jj<np;jj++)
				if (nsh[jj][ii]>0)
					shear[jj][ii] /=nsh[jj][ii];
		
		if (M.iref>0)
		{
			wrf_fits_abs(file,shear,np,np,sxmin,sxmax,symin,symax,M.ref_ra,M.ref_dec);
		}
		else
		{
			wrf_fits(file,shear,np,np,sxmin,sxmax,symin,symax);
		}
		
		
	}
	
	free_square_double(shear,np);
	free_square_int(nsh,np);
}


