#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<math.h>
#include<fonction.h>
#include<constant.h>
#include<dimension.h>
#include<structure.h>
#include<lt.h>

double getLhood0();

struct shear   shm[NAMAX];
struct galaxie multi[NIMAX][NIMAX];
struct z_lim   zlim[NIMAX];
struct z_lim   zlim_s[NIMAX];
struct pot     lmin_s[NLMAX],lmax_s[NLMAX],prec_s[NLMAX];
struct matrix  amplifi_mat[NIMAX][NIMAX],amplifi_matinv[NIMAX][NIMAX];
struct pixlist plo[100000]; /*list of points that compose the polygon around the image to inverse*/

double z_dlsds;
double chi_im,chip,chix,chiy,chia,chil,chis;
double amplifi[NIMAX][NIMAX];
double **imo,**soo,**ero;
double drima;	/* distance ratio between the image to inverse and the main lens*/
double **sm_p;		/* optimization spline map : initial save */

int    nshmap; 
int    optim_z, n_famille;
int    block_s[NLMAX][NPAMAX];
int	 **imuo;
int    nplo;	/*number of points of the polygon around the image to inverse*/

double distmin[NIMAX]; /* List of minimal euclidean distances between 
						  the images of a same familly i*/

int    nwarn;	// warning emitted during image plane chi2


/****************************************************************/
/*		nom:		global				*/
/*		auteur:		Jean-Paul Kneib			*/
/*		date:		10/02/92			*/
/*		place:		Toulouse			*/
/****************************************************************/

void  o_global()
{
	
	/*************  declaration de common et locale ****************/
	
	extern struct g_source	S;
	extern struct g_mode	M;
	extern struct g_image	I;
	extern struct g_grille	G;
	extern struct g_pot	P;
	extern struct pot		lens[],lmin[],lmax[],prec[];
	extern struct ipot	ip;
	extern struct galaxie	arclet[];
	extern struct z_lim	zlim[];
	extern struct MCarlo  	mc;
	extern struct g_pixel	imFrame,ps;
		
	extern int    narclet;
	extern int    block[][NPAMAX];
	extern double **map_p;
	
	double evidence;	// in the bayesian optimisation
	int	 constraints, parameters; // for the computation of dof
	
	register int    i,j;
	
	/**************************************************************/
	
	NPRINTF(stderr,"------- GLOBAL - Lens Optimisation - JPK, March 1994 ------\n");
	/* preparation de l'optimisation */
	readConstraints();
	
	/*-------------------------------------------------------*/
	/* optimisation */
	NPRINTF(stderr,"OPTIMISATION: \n");
	
	// set the evidence and compute the number of dof.
	evidence = 0;
	if( M.inverse==3 ) NPRINTF( stderr, "Bayesys Rate : %lf \n",M.rate);
	
	constraints=getNConstraints();
	parameters=getNParameters();
	NPRINTF(stderr, "Number of contraints : %d\n", constraints);
	NPRINTF(stderr, "Number of free parameters : %d\n", parameters);
	
	if( parameters > constraints )
	{
		NPRINTF( stderr, "ERROR : Too many free parameters.\n Optimisation skipped.\n");
		exit(-1);
		return;
	}
	
	// Start the optimisation engines
	if( M.inverse==3)
		evidence = o_run_bayes();
	else
	{
		/* sauvegarde des parametres d'optimisations */
		
		/* is the potentiel a spline? */
		if (lens[0].type!=10)
		{
			for(i=0;i<G.no_lens;i++)
			{
				lmin_s[i]=lmin[i];
				lmax_s[i]=lmax[i];
				prec_s[i]=prec[i];
				for(j=0;j<NPAMAX;j++)
					block_s[i][j]=block[i][j];
			}
		}
		else
		{
			sm_p=(double **) alloc_square_double (G.nx,G.ny);
			sm_p=map_p;
		}
		
		for(i=0;i<I.nzlim;i++)
			zlim_s[i]=zlim[i];
		
		/* selection/identification des parametres pour la carte de chi2 */
		// if any block[ils][ipx] is < 0, set the ip.map global variable
		o_set_map();
		if( ip.map > 0 ) NPRINTF( stderr, "map dimensions : %d \n",ip.map);
		
		if (M.inverse==1)	/* direct fitting */
		{
			if (mc.optMC == 1)
				o_run_mc();
			else if (mc.optMC == 2)
				o_run_mc0();
			else if (zlim[0].opt==2)
				o_runz_mc(); 
			else if (zlim[0].opt==3)
				o_runz_mc0(); 
			else if (ip.map==0)
				o_run();
			else if (ip.map==1)
				o_run1();
			else if (ip.map==2)
				o_run2();	
		}
		else if (M.inverse==2)	/* potfile fitting */
		{
			if ((P.ircut>1) && (P.isigma>1))
				o_runpot2();
			else if (P.ircut>1)
				o_runpot1(1);
			else if (P.isigma>1)
				o_runpot1(2);
			else
				o_run();
		}
	}
	
	/*-------------------------------------------------------
	 * Reset the lens parameters and print the optimisation results */
	
	/*Fill the chires.dat file*/
	o_chires("chires.dat");
	
	/*Reset the lens parameters*/
	set_res_par();
		
	/*Print the results of optimisation*/
	o_print_res(o_chi(), evidence);
	
//	NPRINTF(stderr,"chi: %.1lf (%.1lf,%.1lf)\n",chi0,chip,chil);	
	
	
	
	
	/*-------------------------------------------------------*/
	/*  resultats  */
	
	if (lens[0].type!=10)
	{
		for(i=0;i<G.no_lens;i++)
		{
			if (lens[i].type>1)
				lens[i].sigma=sqrt(lens[i].b0/6./pia_c2);
			else
				lens[i].sigma=sqrt(lens[i].b0/4./pia_c2);

			lens[i].dlsds=dratio(lens[i].z,S.zs);
		};
	}
	
	if (I.stat!= 0)
		o_stat(narclet,arclet);
	
	if (M.iclean!=0)
	{
		/* 
		*  free the square maps
		*/
	
		free_square_double(soo,ps.ny);
		free_square_double(ero,ps.ny);
		free_square_int(imuo,ps.ny);
	
		free_square_double(imo,imFrame.ny);
	
	};
	NPRINTF(stderr,"------- Lens Optimisation Done -----------\n");
}

// Compute the number of constraints with multiple images
int	getNConstraints()
{
	extern struct g_image I;
	extern int narclet;
	int constraints, i;
		
	constraints = 0;
	for( i = 0; i < I.n_mult; i ++)
		constraints += 2*(I.mult[i] - 1);		
	// Sums up the critical lines
	constraints += 2*I.npcl;

	// arclets
	if( I.stat == 4 || I.stat == 5 )
		constraints += 2*narclet;
	else
		constraints += narclet;
	
	return(constraints);
}

/* Read the different constraints to prepare the optimisation 
 * according to the global variables set in the .par file. 
 */
void readConstraints()
{
	extern struct g_source	S;
	extern struct g_mode	M;
	extern struct g_image	I;
	extern struct g_grille	G;
	extern struct g_pot	P;
	extern struct pot		lens[],lmin[],lmax[],prec[];
	extern struct cline	cl[];	
	extern struct ipot	ip;
	extern struct galaxie	arclet[];
	extern struct z_lim	zlim[];
	extern struct MCarlo  	mc;
	extern struct g_pixel	imFrame,ps;
		
	extern int    narclet;
	extern int    block[][NPAMAX];
	extern double **map_p;

	struct galaxie 	mult[NIMAX*NIMAX];
	int	 np,ntmult;
	int    i;
	FILE	*IN;
	
	if (M.iclean!=0)
	{
		imo=(double **) o_invim(M.zclean,&drima,plo,&nplo);
	
		/*
		*  allocate square map
		*/
	
		soo= (double **) alloc_square_double(ps.ny,ps.nx);
		ero= (double **) alloc_square_double(ps.ny,ps.nx);
		imuo= (int **) alloc_square_int(ps.ny,ps.nx);
	
	}
	
	if (I.shmap!= 0)
	{
		nshmap=0;
		f_shmap(&nshmap,shm,I.shfile);
		if (nshmap<1) 
			I.shmap=0;

		I.drsh=dratio(lens[0].z,I.zsh);
	}
	
	if (I.stat!= 0)
	{
		narclet=0;

		// Test if the number of arclets is lower than NAMAX
		IN = fopen(I.arclet, "r");
		if( wc(IN) > NAMAX )
		{
			fprintf( stderr, "ERROR: Too many arclets in %s. Max allowed %d\n", 
				I.arclet, NAMAX);
			fclose(IN);
			exit(-1);
		}
		fclose(IN);
		
		f_shape(&narclet,arclet,I.arclet);
			
		if (M.seeing>0.)
			cor_seeing(narclet,arclet,M.seeing);
	
		if (narclet<1)
			I.stat=0;
		else
			pro_arclet(narclet,arclet);
			
		if (strcmp(I.nza,""))
			I.drarclet=dratio(lens[0].z,I.zarclet);
	}
	
	if (I.n_mult!= 0)
	{
		ntmult=0;
//		if (I.mult_abs!=0)
//			f_shape_abs(&ntmult,mult,I.multfile);
//		else
			f_shape(&ntmult,mult,I.multfile);
	
		if (M.seeing>0.)
			cor_seeing(ntmult,mult,M.seeing);

		if (ntmult <1)
			I.n_mult=0;
		else
		{
			pro_arclet_m(ntmult,mult);
			o_prep_mult(ntmult,mult);
			dist_min();
		}
	}
	
	if (I.npcl!=0)
	{
		/* preparation de l'optimisation dans le cas de contraintes de ligne */
		/* critiques (cassures d'arcs) */
		np=0;
		for(i=0;i<I.npcl;i++)
		{
			if (cl[i].n !=0)
			{
				cl[i].dlsds=dratio(lens[0].z,cl[i].z);
				cl[np]=cl[i];	// stack all the CL contraints at the beginning of <cl> list
				np++;
			};
		};
		I.npcl=np;	// new number of CL constraints
		NPRINTF(stderr,"INF: critical points: %d\n",I.npcl);
	}
}

// Compute the number of free parameters
int getNParameters()
{
	extern	struct	g_grille	G;
	extern	struct	g_pot		P;
	extern	struct	g_image		I;
	extern	int		block[][NPAMAX];
	extern	int		cblock[];	
	extern	struct	z_lim		zlim[];

	int parameters, ilens, ipx;
		
	parameters = 0;
	for( ilens = 0; ilens < G.no_lens; ilens ++ )
		for( ipx = CX; ipx <= RCUT; ipx ++ )
			if( block[ilens][ipx] != 0 )
				parameters++;
				
	for( ipx = OMEGAM; ipx <= WX; ipx++ )
		if( cblock[ipx] ) parameters++;
	 
	parameters += I.nzlim;
	
	if( P.ircut != 0 ) parameters++;
	if( P.isigma != 0 ) parameters++;
	
	if( I.dsigposAs != -1. ) parameters++;
	if( I.dsigell != -1. ) parameters++;
		
	return(parameters);	
}
