#include<stdio.h>
#include<signal.h>
#include<stdlib.h>
#include<string.h>
#include<math.h>
#include<fonction.h>
#include<constant.h>
#include<dimension.h>
#include<structure.h>
#include "bayesys3.h"
#include "userstr.h"

/****************************************************************/
/*		nom:		o_run_bayes			*/
/*		auteur:		Eric Jullo			*/
/*		date:		01/06   			*/
/*		place:		ESO, Chile			*/
/****************************************************************
 * Optimise the potential parameters using the bayesys3 algorithm.
 * 
 */
typedef void (*sighandler_t)(int);
static void signalReset();
int	optInterrupt;	// Global variable read in bayesapp.c/UserMonitor()

double  o_run_bayes()
{
	extern struct g_mode    M;
	extern struct g_grille  G;
	extern struct g_image   I;
	extern struct g_pot     P;
	extern struct pot       lens[];
	extern struct z_lim     zlim[];
	extern int block[][NPAMAX];
	extern int cblock[NPAMAX];

// Addressing
	CommonStr     Common[1];
	ObjectStr	  *Objects;
	UserCommonStr UserCommon[1];
	int nDim = 0; //# of parameters
 	int ipx;      //index of the current parameter in the Param
 	int i;        //index of the clump in the lens[] global variable
 	int k;        //index of the z_m_limit images to optimise 

	FILE *bayes;
	char name[60];


// Clean the results files
#ifdef DEBUG
	bayes= fopen( "bayes.dbg.dat" , "w" );
#else
	bayes= fopen( "bayes.dat" , "w" );
#endif
	
	fprintf( bayes, "#Nsample\n");
	if( I.dsigposAs != -1 || I.dsigell != -1 )
		fprintf( bayes, "#ln(Lhood)\n");
	else
		fprintf( bayes, "#Chi2\n");
		 
//	fprintf( bayes, "#Evidence\n");
	
// Number of lens parameters in the atom
	for( i = 0; i < G.no_lens; i++ )
		for( ipx = CX; ipx <= PMASS; ipx++ )
			if( block[i][ipx] != 0 )
			{
				nDim++;
				fprintf( bayes, "#%s : %s\n", lens[i].n, getParName(ipx,name,lens[i].type) );
			}

// Number of cosmological parameters in the atom
	for( ipx = OMEGAM; ipx <= WX; ipx++ )
		if( cblock[ipx] != 0 )
		{
			nDim++;
			fprintf( bayes, "#%s\n", getParName(ipx,name,0) );
		}
		
// Number of redshifts to optimise
	for( k = 0; k < I.nzlim; k++ )
		if( zlim[k].opt != 0 )
		{
			nDim++;
			fprintf( bayes, "#Redshift of %s\n", zlim[k].n );
		}
	
// Potfile parameters to optimise
	if( P.ircut != 0 )
	{
		nDim++;
		fprintf( bayes, "#Pot rcut (arcsec)\n");
	}
	if( P.isigma != 0 )
	{
		nDim++;
		fprintf( bayes, "#Pot sigma (km/s)\n");
	}
	
// The noise
	if( I.dsigposAs != -1. )
	{
		nDim++;
		fprintf( bayes, "#SigposArsec (arcsec)\n");
	}
	if( I.dsigell != -1. )
	{
		nDim++;
		fprintf( bayes, "#Sigell (arcsec)\n");
	}
	
	fclose(bayes);

// Initialise BayeSys
	Common->MinAtoms    = 1;          // >= 1
	Common->MaxAtoms    = 1;          // >= MinAtoms, or 0 = infinity
	Common->Alpha       = -1.0;       // +ve for Poisson, -ve for geometric
	Common->ENSEMBLE    = 10;   	  // # objects in ensemble
	if( M.minchi0 > 10 )
		Common->ENSEMBLE    = (int)ceil(M.minchi0);   
	Common->Method      = -1;          // Algorithm method
	Common->Rate        = M.rate;     // Speed of calculation (dimensionless)
	Common->Iseed       = -4321;       // Random seed, -ve is time seed
#ifdef DEBUG
	Common->Iseed	  = 4321;		// Reproductible samples
#endif
	Common->Ndim        = nDim;    	  // dimension = # coordinates
	Common->Valency     = 0;          // # MassInf fluxes per ato
		
// Initialise statistics
	Common->UserCommon = (void*)UserCommon;
	UserCommon->Nsample = 0;
	UserCommon->atoms   = 0.0;
	UserCommon->Nchi2 = 0;
	
	Objects = (ObjectStr *)malloc((unsigned) Common->ENSEMBLE*sizeof(ObjectStr));

// Handle CTRL-C to interrupt the optimisation but not lenstool
	signal(SIGINT, signalReset);
	optInterrupt = 0;

//Run bayesys
	BayeSys3(Common, Objects);

// Reset the SIGINT signal to its default behavior
	signal(SIGINT, SIG_DFL);

	fprintf(stdout, "\n");
	
	/*Set the lens parameters that give the best model and the 1sigma limits*/
	o_set_lens_bayes();

	NPRINTF( stderr, "log(Evidence)          = %lf\n", Common->Evidence );
	NPRINTF( stderr, "Information = -Entropy = %lf\n", Common->Information );
	NPRINTF( stderr, "Calls to chi2() function = %ld\n", UserCommon->Nchi2 );
	
//	free(array);
	/*Free the structures*/
	return( Common->Evidence );
}


static void signalReset()
{
	signal(SIGINT, SIG_DFL);
	optInterrupt = 1;
	fprintf( stderr, "INFO: Optimisation interrupted by CTRL-C\r");
}
