#include<stdlib.h>
#include<fonction.h>
#include<constant.h>
#include<dimension.h>
#include<structure.h>


static void o_set_limit_bayes(int nParam, int nVal, double **array);
/****************************************************************/
/*		nom:		o_set_lens_bayes	*/
/*		auteur:		Eric Jullo			*/
/*		date:		03/05/06   			*/
/*		place:		ESO, Chile			*
 *
 * Set the lens parameters that give the most probable model.
 * 
 * For each parameter of the <bayes.dat> assign the median value
 */
void o_set_lens_bayes() 
{

	
	double   **array;		// contains the bayes.dat data
	int      nParam, nVal;	//size of array
		      	
	// Read the bayes.dat file
	array = readBayesModels(&nParam, &nVal);
	
	if( nVal == 0 )
	{
		fprintf(stderr, "ERROR: bayes.dat file not found.\n");
		exit(-1);
	}

	// set the best model ( code = -4 )
	setBayesModel(-4, nParam, nVal, array);

	// Set the lmin and lmax limits to 1 sigma error
	o_set_limit_bayes(nParam, nVal, array);
	
	free(array);
}

// Set the lmin and lmax values to 1sigma error around the median value
static void o_set_limit_bayes(int nParam, int nVal, double **array)
{
	extern struct g_grille G;
	extern struct g_pot    P;
	extern struct g_cosmo  C;
	extern struct g_image  I;
	extern struct pot      lmin[];
	extern struct pot      lmax[];
	extern struct pot      clmin, clmax;
	extern struct z_lim    zlim[];
	extern int block[][NPAMAX];
	extern int cblock[NPAMAX];

	int      iParam;
	int      i, ipx;
	double   med, std;

	// Set the position of the first physical parameter in bayes.dat
	iParam = IDPARAM1;

	// Set the optimized clumps limits
	for( i = 0; i < G.no_lens; i++ )
	{
		for( ipx = CX; ipx <= PMASS; ipx++ )
			if( block[i][ipx] != 0 )
			{
				block[i][ipx] = 1;	// set uniform prior with bounds at 1 sigma.
				med = median(nVal, array[iParam]);
				std = stddev(nVal, array[iParam++]);
				o_set_lmin( i, ipx, med - std );
				o_set_lmax( i, ipx, med + std );
				
				if( ipx == B0 ) 
				{
					lmin[i].sigma = med - std;
					lmax[i].sigma = med + std;
				}
			}
	}

	// Set the cosmo
	if( cblock[OMEGAM] != 0 )
	{
		cblock[OMEGAM] = 1;	// set uniform prior with bounds at 1 sigma.
		med = median(nVal, array[iParam]);
		std = stddev(nVal, array[iParam++]);
		clmin.omegaM = med-std;
		clmax.omegaM = med+std;
	}
	if( cblock[OMEGAX] != 0 )
	{
		cblock[OMEGAX] = 1;	// set uniform prior with bounds at 1 sigma.
		med = median(nVal, array[iParam]);
		std = stddev(nVal, array[iParam++]);
		clmin.omegaX = med-std;
		clmax.omegaX = med+std;
	}
	if( cblock[WX] != 0 )
	{
		cblock[WX] = 1;	// set uniform prior with bounds at 1 sigma.
		med = median(nVal, array[iParam]);
		std = stddev(nVal, array[iParam++]);
		clmin.wX = med-std;
		clmax.wX = med+std;
	}
	
	// rescale the z_m_limit redshifts
	for( i = 0; i < I.nzlim; i++ )
		if( zlim[i].opt != 0 )
		{
			zlim[ipx].bk = 1; // set uniform prior
			med = median(nVal, array[iParam]);
			std = stddev(nVal, array[iParam++]);			
			zlim[ipx].min = med-std;
			zlim[ipx].max = med+std;
		}
	
	// Set the potfile limits
	if( P.ircut != 0 )
	{
		P.ircut = 1; // set uniform prior
		med = median(nVal, array[iParam]);
		std = stddev(nVal, array[iParam++]);
		P.cut1 = med - std;
		P.cut2 = med + std;
		P.cutkpc1 = P.cut1 * (d0/C.h*distcosmo1(P.zlens));
		P.cutkpc2 = P.cut2 * (d0/C.h*distcosmo1(P.zlens));		
	}	
	if( P.isigma != 0 )
	{
		P.isigma = 1; // set uniform prior
		med = median(nVal, array[iParam]);
		std = stddev(nVal, array[iParam++]);
		P.sigma1 = med - std;
		P.sigma2 = med + std;		
	}
	
	// rescale the noise
	if( I.dsigposAs != -1. )
	{
		std = stddev(nVal, array[iParam++]);
		I.dsigposAs = std;
	}
	if( I.dsigell != -1. )
	{
		std = stddev(nVal, array[iParam++]);
		I.dsigell = std;
	}
}
