#include<stdio.h>
#include<stdlib.h>
#include<float.h>
#include<math.h>
#include<fonction.h>
#include<constant.h>
#include<dimension.h>
#include<structure.h>
#include "lt.h"

/****************************************************************/
/*		nom:		set_lens_par			*/
/*		auteur:		Jean-Paul Kneib			*/
/*		date:		10/02/92			*/
/*		place:		Toulouse			*/
/****************************************************************
 * Initialize the lens parameters after they have been read in the 
 * .par file and in the potfile.
 * 
 * Called after reading the .par file in init_grille()
 * Called iteratively by runpot1() and runpot2().
 * 
 * For this iterations, the number of initializations for each lens 
 * has to be minimum.
 */
void	set_lens()
{
	extern struct g_mode    M;
	extern struct g_grille  G;
	extern struct g_cosmo   C;
	extern struct g_source  S;
	extern struct g_pot     P;
	extern struct pot       lens[];
	extern struct pot       lmin[],lmax[],prec[];
	extern int block[][NPAMAX];

	double GG=10.867;
	
	register int i;

	double d1;

	//*********************************************************************
	// Set the Potfile current and limiting values
	//
	// Scale potfile SIGMA 
	P.sigma = P.sigma1;
	
	// ... and potfile RCUT
	if( P.cut1 == -1.0 && P.cutkpc1 != -1. )
	{
		P.cut1 = P.cutkpc1 / (d0/C.h*distcosmo1(P.zlens));
		P.cut2 = P.cutkpc2 / (d0/C.h*distcosmo1(P.zlens));
	}
	P.cut = P.cut1;
	
	// ... and potfile RCORE
	if( P.core == -1.0 && P.corekpc != -1 )
		P.core = P.corekpc / (d0/C.h*distcosmo1(P.zlens));
	
	//************************************************************
	// Set the parameters of all clumps
	// 
	// Scale the potfile RCORE ...
	for( i = G.nplens ; i < G.nlens ; i++ )
		lens[i].rc = P.core*pow(10.,0.4*(P.mag0-lens[i].mag)/2.);

	scale_pot(); // ...and RCUT and SIGMA of the clumps.

	/*
	* SET THE CLUMPS PARAMETERS
	*/
	for( i = 0 ; i < G.nlens ; i++ ) 
	{
		// Converting distance in kpc to arcsec.
		d1 = d0 / C.h * distcosmo1( lens[i].z );
		
	
		// Compute the DLS/DS ratio for all clumps.
		lens[i].dlsds = dratio( lens[i].z, S.zs );
		
		// Set rcore value in kpc or in arcsec.
		if( lens[i].rckpc != 0. )
			lens[i].rc = lens[i].rckpc /d1;
		else 
			lens[i].rckpc = lens[i].rc * d1;

		// Set rcut value in kpc or in arcsec.   
		if( lens[i].rcutkpc != DBL_MAX )
			lens[i].rcut = lens[i].rcutkpc / d1;
		else if( lens[i].rcut != DBL_MAX )
			lens[i].rcutkpc = lens[i].rcut * d1;

		// Set the Mass to Light (mtol) property.
		/* the 1.5 factor come from the fact that we used 6pi\sigma^2 instead of 
		 4pi\sigma^2 - still don't understand why we need the 2.5 factor */
		if( lens[i].lum != 0 )
			lens[i].mtol = 2.5 * 1.5 * PI / GG *
				(lens[i].sigma/1000)*(lens[i].sigma/1000)*
				lens[i].rcutkpc/lens[i].lum;	

		/*
		*  elliptical parameters epot and q
		*/
		if( lens[i].type == 0 || lens[i].type == 2 )
		{
			lens[i].emass=0.;
			lens[i].epot=0.;
			lens[i].theta=0.;
			lens[i].type++;
		}
		else if( lens[i].type ==  8  || 
		         lens[i].type == -2  || 
		         ( lens[i].type > 80 && lens[i].type < 90 ) )
		{
			if( lens[i].emass == -1 && lens[i].epot != 0. )
			// emass is (a2-b2)/(a2+b2)
				lens[i].emass=2.*lens[i].epot/(1.+lens[i].epot*lens[i].epot);
			else if( ( lens[i].emass == -1 && lens[i].epot == 0. ) ||
			         lens[i].emass == 0. )
				lens[i].epot=0.0001;
			else
			// epot is (a-b)/(a+b)
				lens[i].epot=(1.-sqrt(1-lens[i].emass*lens[i].emass))/lens[i].emass;
		}
		else
		{
			if ( lens[i].emass == -1 )
				lens[i].emass=3.*lens[i].epot;
			else
				lens[i].epot=lens[i].emass/3.;
		    	
		};
		
		/*
		*  dynamical parameters ----------------------------------------------------
		*/
		if( lens[i].type == 12 && lens[i].alpha != 0 )
			read_lenstable();		
		
		set_dynamics(i);
	}
	
	/*
	* SET THE LIMITS PARAMETERS
	*/
	for( i = 0 ; i < G.no_lens ; i++ )
	{
		lmin[i].type=lmax[i].type=prec[i].type=lens[i].type;

		/* converting distance in kpc to arcsec */
		d1 = d0 / C.h*distcosmo1(lens[i].z);
		
		// set RCORE value, lmin, lmax and prec in kpc or in arcsec
		if( lmin[i].rckpc != 0. )
			lmin[i].rc = lmin[i].rckpc / d1;
		else 
			lmin[i].rckpc = lmin[i].rc * d1;
		    
		if( lmax[i].rckpc != 0. )
			lmax[i].rc = lmax[i].rckpc / d1;
		else 
			lmax[i].rckpc = lmax[i].rc * d1;
		    
		if( prec[i].rckpc!=0. )
			prec[i].rc = prec[i].rckpc / d1;
		else 
			prec[i].rckpc = prec[i].rc * d1;
		
		// set RCUT value, lmin, lmax and prec in kpc or in arcsec        
		if( lmin[i].rcutkpc != 0. )
			lmin[i].rcut = lmin[i].rcutkpc / d1;
		else 
			lmin[i].rcutkpc = lmin[i].rcut * d1;
		    
		if( lmax[i].rcutkpc != 0. )
			lmax[i].rcut = lmax[i].rcutkpc / d1;
		else 
			lmax[i].rcutkpc = lmax[i].rcut * d1;
		    
		if( prec[i].rcutkpc != 0. )
			prec[i].rcut = prec[i].rcutkpc / d1;
		else 
			prec[i].rcutkpc = prec[i].rcut * d1;

		/*
		* elliptical parameters
		*/
		if( lens[i].type==0 || lens[i].type==2 )
			// Circular SIS
			block[i][EPOT]=0;

	
		/*
		*  dynamical parameters
		*/
		if (lens[i].type==1 || lens[i].type==-1)
		{
			lmin[i].b0=4.*pia_c2*lmin[i].sigma*lmin[i].sigma;
			
			if( M.inverse == 3 && block[i][B0] == 3 )
				lmax[i].sigma = 8.*pia_c2*lmin[i].sigma*lmax[i].sigma;
			else			
				lmax[i].b0=4.*pia_c2*lmax[i].sigma*lmax[i].sigma;
				
			prec[i].b0=4.*pia_c2*prec[i].sigma*prec[i].sigma;
		}
		else if (lens[i].type==7)
		{
			lmin[i].b0=4.*RTA*GM_c2*
			lmin[i].masse/(D0/C.h*distcosmo1(lens[i].z));
			lmax[i].b0=4.*RTA*GM_c2*lmax[i].masse/(D0/C.h*distcosmo1(lens[i].z));
			prec[i].b0=4.*RTA*GM_c2*
			prec[i].masse/(D0/C.h*distcosmo1(lens[i].z));
		}
		else if (lens[i].type==9)
		{
			lmin[i].b0=lmin[i].pmass/cH2piG/C.h*distcosmo1(lens[i].z);
			lmax[i].b0=lmax[i].pmass/cH2piG/C.h*distcosmo1(lens[i].z);
			prec[i].b0=prec[i].pmass/cH2piG/C.h*distcosmo1(lens[i].z);
		}
		else if (lens[i].type==5)
		{
			lmin[i].b0=18.*RTA*lmin[i].sigma*lmin[i].sigma/vol/vol;
			lmax[i].b0=18.*RTA*lmax[i].sigma*lmax[i].sigma/vol/vol;
			prec[i].b0=18.*RTA*prec[i].sigma*prec[i].sigma/vol/vol;
		}
		else if( lens[i].type == 13 ) //Sersic : Keep the limits as they are
		{
			d1 = cH0_4piG*C.h/distcosmo1(lens[i].z);	// in 10^12 Msol/kpc^2
			lmin[i].b0=lmin[i].sigma*1e-12/d1;
			lmax[i].b0=lmax[i].sigma*1e-12/d1;
			prec[i].b0=prec[i].sigma*1e-12/d1;
		}
		else
		{
			lmin[i].b0=6.*pia_c2*lmin[i].sigma*lmin[i].sigma;
			lmax[i].b0=6.*pia_c2*lmax[i].sigma*lmax[i].sigma;
			prec[i].b0=6.*pia_c2*prec[i].sigma*prec[i].sigma;
		}
	}
}

void set_dynamics(int i)
{
	extern struct g_mode   M;
	extern struct g_grille G;
	extern struct g_cosmo  C;
	extern struct pot      lens[];
	extern int block[][NPAMAX];
	register int ii,jj;
	double test;
	
	extern double *v_xx;
	extern double *v_yy;
	extern double **map_p;
	extern double **tmp_p;
	extern double **map_axx;
	extern double **map_ayy;

	double **tmpf;
	char	mode[20],nature[20],type[20],comment[1024];
	struct pot *ilens;
	
	ilens = &lens[i];
	
	switch(ilens->type)
	{
		case(1): // SIS
			ilens->b0=4.*pia_c2*ilens->sigma*ilens->sigma;
			ilens->ct=ilens->b0*ilens->dlsds;
			ilens->cr=0.;
			break;
		case(12): // NFW
			// rescale from c,mvir,rhos --> sigmas (km/s), rs (arcsec)
			if( ilens->beta != 0 && ilens->masse != 0 )
			{
				e_nfw_cm200_sigrs( ilens->beta, ilens->masse, 
					&ilens->sigma, &ilens->rckpc );
				ilens->rc = ilens->rckpc / (d0/C.h*distcosmo1(ilens->z));
			} 
			else if( ilens->beta != 0 && ilens->rcut != DBL_MAX )
			{
				ilens->rcutkpc = ilens->rcut * (d0/C.h*distcosmo1(ilens->z));
				e_nfw_cr200_sigrs( ilens->beta, ilens->rcutkpc,
					&ilens->sigma, & ilens->rckpc );
				ilens->rc = ilens->rckpc / (d0/C.h*distcosmo1(ilens->z));
			}
			else if( ilens->beta != 0 && ilens->rc != 0 )
			{
				ilens->rckpc = ilens->rc * (d0/C.h*distcosmo1(ilens->z));
				e_nfw_crs2sig( ilens->beta, ilens->rckpc, &ilens->sigma );
			}
			else if( ilens->rc != 0 && ilens->masse != 0 )
			{
				ilens->rckpc = ilens->rc * (d0/C.h*distcosmo1(ilens->z));
				e_nfw_rsm200_sigrs( ilens->rckpc, ilens->masse,	&ilens->sigma);
			}
			else if( ilens->rc != 0 && ilens->rcut != DBL_MAX )
			{
				ilens->rckpc = ilens->rc * (d0/C.h*distcosmo1(ilens->z));
				ilens->rcutkpc = ilens->rcut * (d0/C.h*distcosmo1(ilens->z));
				e_nfw_rsr200_sigrs( ilens->rckpc, ilens->rcutkpc, &ilens->sigma);
			}

			ilens->b0=6.*pia_c2*ilens->sigma*ilens->sigma;
			
			break;
		case(13): // Sersic
			// here b0 is kappa(Re) = Sigma_e/Sigma_crit // cH0_4piG in 10^12 Msol/kpc^2 
			ilens->b0=ilens->sigma*1e-12/(cH0_4piG*C.h/distcosmo1(ilens->z));
			break;
		case(14): // External shear
			break;
		case(-1): //SIE
			ilens->b0=4.*pia_c2*ilens->sigma*ilens->sigma;
			ilens->ct=ilens->b0*ilens->dlsds;
			ilens->cr=0.;
			break;
		case(-2): 
//			NPRINTF(stderr,"Clump %d: True Elliptical BBS model\n",i);
//			NPRINTF(OUT,"-------- Clump %d: True Elliptical BBS model \n",i);
			ilens->b0=4.*pia_c2*ilens->sigma*ilens->sigma;
			ilens->ct=ilens->b0*ilens->dlsds;
			ilens->cr=0.;
			break;
		case(7):
//			NPRINTF(stderr,"Clump %d: Point Masse \n",i);
//			NPRINTF(OUT,"Clump %d: Point Masse \n",i);
			ilens->b0=4.*RTA*GM_c2*
				ilens->masse/(D0/C.h*distcosmo1(ilens->z));
			ilens->ct=sqrt(ilens->b0*ilens->dlsds);
			ilens->cr=0.;
			break;
		case(9):
//			NPRINTF(stderr,"Clump %d: Plan Masse \n",i);
//			NPRINTF(OUT,"Clump %d: Plan Masse \n",i);
			ilens->b0=ilens->pmass/cH2piG/C.h*distcosmo1(ilens->z);
			ilens->ct=0.;
			ilens->cr=0.;
			break;
		case(5):
//			NPRINTF(stderr,"Clump %d: Hubble Modified Law \n",i);
//			NPRINTF(OUT,"Clump %d: Hubble Modified Law \n",i);
			ilens->b0=18.*RTA*ilens->sigma*ilens->sigma/vol/vol;
			/*
			ilens->phi0=ilens->b*ilens->rc;
			phi0=ilens->phi0; coeur=ilens->rc;
			ilens->ct=iter(ilens->phi0,ilens->rc,ilens->rc);
			ilens->cr=zero(0.,ilens->ct,fz_mhl_cr);
			*/
			 break;
		case(8):
//			NPRINTF(stderr,"Clump %d: PIEMD Kovner\n",i);
//			NPRINTF(OUT,"Clump %d: PIEMD Kovner\n",i);
			ilens->b0=6.*pia_c2*ilens->sigma*ilens->sigma;
			break;
		case(81):
/*			NPRINTF(stderr,"Clump %d: trunc. PIEMD Kovner:",i);
			NPRINTF(OUT,"Clump %d: PIEMD Kovner, truncated\n",i);
			NPRINTF(OUT," Total mass: %lf(cor) %lf (10^12 M_sol)\n",
				4*M_PI/3*M_PI/GG*(ilens->sigma/1000)*(ilens->sigma/1000)*
				ilens->rcut*(d0/C.h*distcosmo1(ilens->z)),
				1.5*M_PI/GG*(ilens->sigma/1000)*(ilens->sigma/1000)*
			ilens->rcut*(d0/C.h*distcosmo1(ilens->z)) );
			NPRINTF(OUT," rcut:%.2lf(\") %.2lf(kpc)\n",
				ilens->rcut,ilens->rcut*(d0/C.h*distcosmo1(ilens->z)) );
*/			
			ilens->b0=6.*pia_c2*ilens->sigma*ilens->sigma;
			break;
			
		case(82):
/*			NPRINTF(stderr,"Clump %d: PIEMD Kovner, shallow center\n",i);
			NPRINTF(OUT,"Clump %d: PIEMD Kovner, shallow center\n",i);
			NPRINTF(OUT," Steep radius:%.2lf(\") %.2lf(kpc)\n",
				ilens->rcut,ilens->rcut*(d0/C.h*distcosmo1(ilens->z)) );
*/					
			ilens->b0=6.*pia_c2*ilens->sigma*ilens->sigma;
			
			break;
		case(83):
/*			NPRINTF(stderr,"Clump %d: EMD Kovner, 3/2\n",i);
			NPRINTF(OUT,"Clump %d: PIEMD Kovner, shallow center\n",i);
			NPRINTF(OUT," Steep radius:%.2lf(\") %.2lf(kpc)\n",
				ilens->rcut,ilens->rcut*(d0/C.h*distcosmo1(ilens->z)) );
*/					
			ilens->b0=6.*pia_c2*ilens->sigma*ilens->sigma;
			break;
		case(84):
/*			NPRINTF(stderr,"Clump %d: EMD Kovner, 0.5a-0.5s+1.5s\n",i);
			NPRINTF(OUT,"Clump %d: PIEMD Kovner, 0.5a-0.5s+1.5s\n",i);
			NPRINTF(OUT," Steep radius:%.2lf(\") %.2lf(kpc)\n",
				ilens->rcut,ilens->rcut*(d0/C.h*distcosmo1(ilens->z)) );
*/					
			ilens->b0=6.*pia_c2*ilens->sigma*ilens->sigma;
			break;
		case(85):
/*			NPRINTF(stderr,"Clump %d: EMD Kovner, 1\n",i);
			NPRINTF(OUT,"Clump %d: PIEMD Kovner, 1a\n",i);
			NPRINTF(OUT," Steep radius:%.2lf(\") %.2lf(kpc)\n",
				ilens->rcut,ilens->rcut*(d0/C.h*distcosmo1(ilens->z)) );
*/					
			ilens->b0=6.*pia_c2*ilens->sigma*ilens->sigma;
			break;
		case(86):
/*			NPRINTF(stderr,"Clump %d: EMD Kovner, 1a-1s\n",i);
			NPRINTF(OUT,"Clump %d: PIEMD Kovner, 1a-1s\n",i);
			NPRINTF(OUT," Steep radius:%.2lf(\") %.2lf(kpc)\n",
				ilens->rcut,ilens->rcut*(d0/C.h*distcosmo1(ilens->z)) );
*/					
			ilens->b0=6.*pia_c2*ilens->sigma*ilens->sigma;
			break;
		case(87):
/*			NPRINTF(stderr,"Clump %d: EMD Kovner, 1a-1s+0.5a-0.5s\n",i);
			NPRINTF(OUT,"Clump %d: PIEMD Kovner, 1a-1s+0.5a-0.5s\n",i);
			NPRINTF(OUT," Steep radius:%.2lf(\") %.2lf(kpc)\n",
				ilens->rcut,ilens->rcut*(d0/C.h*distcosmo1(ilens->z)) );
*/					
			ilens->b0=6.*pia_c2*ilens->sigma*ilens->sigma;
			break;
		case(88):
/*			NPRINTF(stderr,"Clump %d: EMD Kovner, 1a-1s+1.5s\n",i);
			NPRINTF(OUT,"Clump %d: PIEMD Kovner, 1a-1s+1.5s\n",i);
			NPRINTF(OUT," Steep radius:%.2lf(\") %.2lf(kpc)\n",
				ilens->rcut,ilens->rcut*(d0/C.h*distcosmo1(ilens->z)) );
*/					
			ilens->b0=6.*pia_c2*ilens->sigma*ilens->sigma;
			break;
		case(89):
/*			NPRINTF(stderr,"Clump %d: EMD Kovner, 1a-1s+0.5a-0.5s\n",i);
			NPRINTF(OUT,"Clump %d: PIEMD Kovner, 1a-1s+0.5a-0.5s\n",i);
			NPRINTF(OUT," Steep radius:%.2lf(\") %.2lf(kpc)\n",
				ilens->rc*ilens->beta,ilens->rc*ilens->beta*(d0/C.h*distcosmo1(ilens->z)) );
*/					
			ilens->b0=6.*pia_c2*ilens->sigma*ilens->sigma;
			break;
		case(10):
//			NPRINTF(stderr,"Clump %d: Spline Potential\n",i);
//			NPRINTF(OUT,"Clump %d: Spline Potential\n",i);
			
			/* get the potential map */
			
			tmpf=(double **)rdf_ipx(G.splinefile,&G.ny,&G.nx,
				type,mode,nature,comment,
				&G.xmin,&G.xmax,&G.ymin,&G.ymax);
			
			map_p=(double **) alloc_square_double(G.nx,G.ny);
			tmp_p=(double **) alloc_square_double(G.nx,G.ny);
			
			for(ii=0;ii<G.nx;ii++)
				for(jj=0;jj<G.ny;jj++)
					map_p[ii][jj]=tmpf[jj][ii];
			
			free_square_double(tmpf,G.nx);
			
			G.dx=(G.xmax-G.xmin)/(G.nx-1);
			G.dy=(G.ymax-G.ymin)/(G.ny-1);
			
			NPRINTF(stderr,"COMP: vecteurs x & y\n");
			v_xx=(double *) alloc_vector_double(G.nx);
			v_yy=(double *) alloc_vector_double(G.ny);
			for(ii=0;ii<G.nx;ii++)
				v_xx[ii]=G.xmin+ii*G.dx;
				
			for(ii=0;ii<G.ny;ii++)
				v_yy[ii]=G.ymin+ii*G.dy;
			
			/* compute the derivatives map */
			
			map_axx=(double **) alloc_square_double(G.nx,G.ny);
			map_ayy=(double **) alloc_square_double(G.nx,G.ny);
			
			sp_set(map_p,G.nx,G.ny,map_axx,map_ayy);
			
			NPRINTF(stderr,"Clump %d: done\n",i);
			break;
		default:
//			NPRINTF(stderr,"Clump %d: Pseudo-Elliptical Potential with Core Radius\n",i);
//			NPRINTF(OUT,"Clump %d: Pseudo-Elliptical Potential with Core Radius\n",i);
			ilens->b0=6.*pia_c2*ilens->sigma*ilens->sigma;
			test=ilens->dlsds*ilens->dlsds*ilens->b0*ilens->b0-ilens->rc*ilens->rc;
			
			if(test>0.)
			{
				 ilens->ct=sqrt(test);
				 ilens->cr=sqrt(pow(ilens->b0*ilens->dlsds,2./3.)*
					pow(ilens->rc,4./3.)-ilens->rc*ilens->rc);
			}
			else
				ilens->ct=ilens->cr=0.;
			
			if (ilens->type>20)
				updatecut(i);
			
			break;
	}
}
