#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<math.h>
#include<float.h>
#include<fonction.h>
#include<constant.h>
#include<dimension.h>
#include<structure.h>
 
/****************************************************************/
/*                 Program  : grille				*/
/*                 Version  : 1 mai 1992			*/
/*                 Location : Obs. Toulouse			*/
/*                 Auteur   : jean-paul				*/
/*	EJ 30/11/05 Modify indentation and add void flire			*/
/*  EJ 2/12/05  Rearrange the coef in rcut/sigma... equations*/
/****************************************************************/


static void getRADEC(char *line, 
			  int *iref, double *ra, double *dec );

static void convertXY( double *x, double *y, 
 						int iref, double ra, double dec );

int set_potfile(int nplens)
{
	extern struct g_mode   M;
	extern struct g_grille G;
	extern struct pot	     lens[];
	extern struct g_pot    P;
	int	  i;
	double  aa,bb;
	char	  line[128];
	FILE	  *POT;

	double  ra, dec;
	int     iref;
	
	iref = 0;	// Default, wcs coordinates
	i=nplens;

	// Open the potfile	
	NPRINTF(stderr,"READ: %s %d\n",P.potfile,nplens);

	POT=fopen(P.potfile,"r");
	
	if( POT == NULL || ferror(POT) )
	{
		fprintf( stderr, "ERROR: Error reading %s\n", P.potfile);
		exit(-1);
	}       	
	
	/* clump type file */
	if( P.ftype==2 || P.ftype==4 )
	{
		while( !feof(POT) && !ferror(POT) )
		{
			fgets(line, 128, POT);			
			if( strstr(line, "#REFERENCE" ) != NULL )
			{
				//NPRINTF( stderr, "%s\n", line );
				getRADEC(line, &iref, &ra, &dec );
			}
			else if( sscanf( line,"%d%lf%lf%lf%lf%lf%lf%lf%lf",
            	      &lens[i].type,&lens[i].C.x,&lens[i].C.y,
			      &lens[i].emass,&lens[i].theta,&lens[i].rckpc,
			      &lens[i].rcutkpc,&lens[i].sigma,&lens[i].z ) == 9 )
			{
				// Convert input from relative to absolute coordinates if needed
				convertXY( &lens[i].C.x, &lens[i].C.y, iref, ra, dec);
				
				// Give an ID to each clump
				line[0] = '\0';
				if( i < G.no_lens )
					strcpy( line, "O" );
					
				sprintf(lens[i].n, "%s%d", line, i+1);
				
				// Initialize the lens parameters
				lens[i].epot=0.;
				lens[i].theta*=DTR;
				lens[i].alpha=0;
				 
				// convert to output relative coordinates
				if( M.iref == 1 || M.iref == 3 )
				{		
					lens[i].C.x -=M.ref_ra;
					lens[i].C.x *=-3600*cos(M.ref_dec*DTR);
					lens[i].C.y -=M.ref_dec;
					lens[i].C.y *=3600;
				} 
				else if( M.iref == 2 )
				{
					lens[i].C.x -= M.ref_ra;
					lens[i].C.y -= M.ref_dec;
				}
				
				i++;								
			}	
		}
	} /*end of if clump type file P.ftype==2*/
	
	/* photometric type file */
	else if(P.ftype==1 || P.ftype==3)
	{
		// Check if the scaling relation is refined
		if( P.sigma1 == -1 )
		{
			fprintf(stderr," ERROR: potfile: sigma not defined\n");
			exit(-1);
		}
			
		if( P.cutkpc1 == -1 && P.cut1 == -1 )
		{
			fprintf(stderr," ERROR: potfile: cut length not defined\n");
			exit(-1);
		}
	
		if( P.corekpc == -1 && P.core == -1 )
		{
			fprintf(stderr," ERROR: potfile: core length not defined\n");
			exit(-1);
		}


		// Read the potfile
		while( !feof(POT) && !ferror(POT) )
		{
			fgets(line, 128, POT);
			if( strstr(line, "#REFERENCE" ) != NULL )
			{
				//NPRINTF( stderr, "%s\n", line );
				getRADEC( line, &iref, &ra, &dec );
			}
			else if( sscanf( line,"%s%lf%lf%lf%lf%lf%lf%lf",
				lens[i].n,&lens[i].C.x,&lens[i].C.y,
				&aa,&bb,&lens[i].theta,&lens[i].mag,&lens[i].lum ) == 8)
			{
				if( lens[i].n[0] != '#' ) 
				{
					// Convert input from relative to absolute coordinates if needed
					convertXY( &lens[i].C.x, &lens[i].C.y, iref, ra, dec );

					// Eventually modify the ID of the clump
					if( i < G.no_lens )
					{
						strcpy( line, lens[i].n );
						sprintf(lens[i].n, "O%s", line);
					}

					// convert to output relative coordinates
					if (  M.iref == 1 || M.iref == 3 )
					{
						lens[i].C.x -=M.ref_ra;
						lens[i].C.x *=-3600*cos(M.ref_dec*DTR);
						lens[i].C.y -=M.ref_dec;
						lens[i].C.y *=3600;
					}
					else if( M.iref == 2 )
					{
						lens[i].C.x -=M.ref_ra;
						lens[i].C.y -=M.ref_dec;
					}

					// Initialize the lens parameters
					lens[i].type=P.type;
					lens[i].z=P.zlens;
					lens[i].emass=(aa*aa-bb*bb)/(aa*aa+bb*bb);
					if( lens[i].emass < 0 )
					{
						fprintf( stderr, "ERROR: The potfile clump %s has a negative ellipticity.\n",
							lens[i].n );
						exit(-1);
					}
					lens[i].theta*=DTR;
					lens[i].alpha=0;
					
		    			// Initialize sigma, rcore and rcut to their default values 
		    			// (for coherence reasons with the rest of the code, 
		    			// eg. with set_lens()).
		    			// Real values are set in o_scale_pot().
		    			lens[i].sigma = 0.;
		    			lens[i].rc = lens[i].rckpc = 0.;
		    			lens[i].rcut = lens[i].rcutkpc = DBL_MAX;
		    			
					i++;
				}
			}
		} // endof while loop over the potfile lines.		
	} /* end of photometric type file P.ftype is 1 or 3 */

	fclose(POT);
	 
	/*
	return the number of clump read in the potfile -----------------------------
	*/
	return(i-nplens);

}

/* Return the ra and dec values in degrees and scaling factors
 * parameters :
 * - line : the #REFERENCE line
 */
static void getRADEC(char *line, 
			  int *iref, double *ra, double *dec )
{
	double	ss0,tt0;
	int		hh0,mm0,dd0,nn0;	
	
	char	ref1[20],ref2[20];	
	
	*iref = *ra = *dec = 0;
		
	if( sscanf(line, "%*s%d%s%s", 
			iref, ref1, ref2) == 3 )
	{
		// Convert ra and dec strings to double
		if( *iref == 1 )
		{
			sscanf(ref1,"%d:%d:%lf",&hh0,&mm0,&ss0);
			sscanf(ref2,"%d:%d:%lf",&dd0,&nn0,&tt0);

			*ra=((double)(hh0)+((double)(mm0))/60.+ss0/3600.)*15.;
		
			if (dd0<0)
				*dec=((double)(dd0))-((double)(nn0))/60-tt0/3600;
			else
				*dec=((double)(dd0))+((double)(nn0))/60+tt0/3600;
		}
		else if( *iref == 3 || *iref == 2 )
		{
			sscanf(ref1,"%lf",ra);
			sscanf(ref2,"%lf",dec);
		}
	}
}

/* Convert an input from relative coordinates to absolute coordinates
 */
static void convertXY( double *x, double *y, 
 						int iref, double ra, double dec )
{
	extern	struct g_mode	M;
	extern struct	g_pot	P;
	
	// Convert the input values to absolute WCS coordinates
	if( iref == 1 || iref == 3 )
	{
		*x /=-3600.*cos(dec*DTR);
		*x += ra;
		*y /=3600.;
		*y +=dec;
	} 
	else if( iref == 2 )  // image coordinates
	{
		*x += ra;
		*y += dec;
	}
	
	// For compatibility with the former syntax
	if( iref == 0 && P.ftype == 1 )
	{
		*x /=-3600.*cos(M.ref_dec*DTR);
		*x += M.ref_ra;
		*y /= 3600.;
		*y += M.ref_dec;
	}
}

/*
 * Scale the sigma and rcut of a potfile clump according to the potfile parameters
 */
void scale_pot()
{
	extern struct g_pot	P;
	extern struct g_grille	G;
	extern struct pot		lens[];
	
	int i;
	
	// loop over the potfile clumps to scale
	for( i = G.nplens; i < G.nlens; i++ )
		if( lens[i].mag != 0 )
		{
			lens[i].sigma=P.sigma*
				pow(10.,0.4*(P.mag0-lens[i].mag)/P.vdslope1);
				
			// Convert sigma to b0
			set_dynamics(i);
		
    		/* The factor of 2 so that with slope1 = 4, we have
    		 * 2/slope1=1/2, then Brainerd, Blandford, Smail, 1996 */
			lens[i].rcut=P.cut*
				pow(10.,0.4*(P.mag0-lens[i].mag)*2/P.slope1);		
		}
		
}

/*
 * Scale the sigma and rcut of a potfile clump according to the potfile parameters
 * AND ... reset the image.grad(2) parameters for the optimisation.
 */
void o_scale_pot()
{
	extern struct g_image	I;
	extern struct galaxie	multi[][NIMAX];
	extern struct galaxie	arclet[NAMAX];
	extern int narclet;	
	
	int i,k;
	
	scale_pot();

	// reset the grad and grad2 arclet temporary structure for optimisation
	for( i=0; i < I.n_mult; i++ )
		for( k=0; k < I.mult[i]; k++ )
		{
			multi[i][k].grad2.a = 0;
			multi[i][k].grad2.c = 0;
			multi[i][k].grad.x = 0;
			multi[i][k].grad.y = 0;
		}

	for( i = 0 ; i< narclet ; i++ )
	{
		arclet[i].grad2.a = arclet[i].grad2.c = 0;
		arclet[i].grad.x = arclet[i].grad.y = 0;
	}

}  
